@@ -20,6 +20,14 @@
         } while (0)
 #endif
 
+#if PY_VERSION_HEX < 0x02050000
+typedef int Py_ssize_t;
+#define PY_SSIZE_T_MAX INT_MAX
+#define PY_SSIZE_T_MIN INT_MIN
+typedef inquiry lenfunc;
+#endif
+
+
 /* from gobjectmodule.c */
 extern struct _PyGObject_Functions pygobject_api_functions;
 #define pyg_block_threads()   G_STMT_START { \
@@ -51,6 +59,17 @@ extern struct _PyGObject_Functions pygobject_api_functions;
     } G_STMT_END
 
 
+#ifndef Py_CLEAR /* since Python 2.4 */
+# define Py_CLEAR(op)				\
+        do {                            	\
+                if (op) {			\
+                        PyObject *tmp = (PyObject *)(op);	\
+                        (op) = NULL;		\
+                        Py_DECREF(tmp);		\
+                }				\
+        } while (0)
+#endif
+
 extern GType PY_TYPE_OBJECT;
 
 extern GQuark pygboxed_type_key;
@@ -64,7 +83,9 @@ extern GQuark pygobject_class_key;
 extern GQuark pygobject_wrapper_key;
 extern GQuark pygpointer_class_key;
 extern GQuark pygobject_has_updated_constructor_key;
+extern GQuark pygobject_instance_data_key;
 
+void     pygobject_data_free  (PyGObjectData *data);
 void     pyg_destroy_notify   (gpointer     user_data);
 gboolean pyg_handler_marshal  (gpointer     user_data);
 gboolean pyg_error_check      (GError     **error);
@@ -80,6 +101,8 @@ PyObject *pyg_integer_richcompare(PyObject *v,
                                   PyObject *w,
                                   int op);
 
+gboolean pyg_gerror_exception_check(GError **error);
+
 /* from pygtype.h */
 extern PyTypeObject PyGTypeWrapper_Type;
 
@@ -122,7 +145,6 @@ extern PyTypeObject PyGInterface_Type;
 extern PyTypeObject PyGProps_Type;
 extern PyTypeObject PyGPropsDescr_Type;
 extern PyTypeObject PyGPropsIter_Type;
-
 void          pygobject_register_class   (PyObject *dict,
 					  const gchar *type_name,
 					  GType gtype, PyTypeObject *type,
@@ -136,7 +158,7 @@ void          pygobject_watch_closure    (PyObject *self, GClosure *closure);
 void          pygobject_register_sinkfunc(GType type,
 					  void (* sinkfunc)(GObject *object));
 int           pyg_type_register          (PyTypeObject *class,
-					  const gchar *typename);
+					  const gchar *type_name);
 
 /* from pygboxed.c */
 extern PyTypeObject PyGBoxed_Type;
@@ -165,7 +187,7 @@ extern PyTypeObject PyGFlags_Type;
 #define PyGFlags_Check(x) (g_type_is_a(((PyGFlags*)x)->gtype, G_TYPE_FLAGS))
 			   
 extern PyObject * pyg_flags_add        (PyObject *   module,
-					const char * typename,
+					const char * type_name,
 					const char * strip_prefix,
 					GType        gtype);
 extern PyObject * pyg_flags_from_gtype (GType        gtype,
@@ -182,7 +204,7 @@ typedef struct {
 extern PyTypeObject PyGEnum_Type;
 
 extern PyObject * pyg_enum_add        (PyObject *   module,
-				       const char * typename,
+				       const char * type_name,
 				       const char * strip_prefix,
 				       GType        gtype);
 extern PyObject * pyg_enum_from_gtype (GType        gtype,
@@ -229,6 +251,11 @@ typedef struct
     PyObject *fd_obj;
 } PyGPollFD;
 
+  /* Data that belongs to the GObject instance, not the Python wrapper */
+struct _PyGObjectData {
+    PyTypeObject *type; /* wrapper type for this instance */
+    GSList *closures;
+};
 
 /* pygoption.c */
 extern PyTypeObject PyGOptionContext_Type;
@@ -253,4 +280,22 @@ PyObject * pyg_option_context_new (GOptionContext * context);
 PyObject * pyg_option_group_new (GOptionGroup * group);
 GOptionGroup *pyg_option_group_transfer_group(PyGOptionGroup *self);
 
+extern GHashTable *custom_type_registration;
+void pyg_type_register_custom_callback(const gchar *type_name,
+				       PyGTypeRegistrationFunction callback,
+				       gpointer data);
+PyTypeObject * pyg_type_get_custom(const gchar *name);
+GType _pyg_type_from_name(const gchar *name);
+
+/* pygobject.c */
+extern PyTypeObject PyGObjectWeakRef_Type;
+
+static inline PyGObjectData *
+pyg_object_peek_inst_data(GObject *obj)
+{
+    return ((PyGObjectData *) 
+            g_object_get_qdata(obj, pygobject_instance_data_key));
+}
+
+
 #endif
