@@ -1,13 +1,16 @@
-# -*- Mode: Python -*-
+#!/usr/bin/env python
 
+import exceptions
+import os
+import sys
+import select
 import unittest
 
-import glib
+from common import gobject
 
-
-class Idle(glib.Idle):
+class Idle(gobject.Idle):
     def __init__(self, loop):
-        glib.Idle.__init__(self)
+        gobject.Idle.__init__(self)
         self.count = 0
         self.set_callback(self.callback, loop)
 
@@ -15,10 +18,9 @@ class Idle(glib.Idle):
         self.count += 1
         return True
 
-
-class MySource(glib.Source):
+class MySource(gobject.Source):
     def __init__(self):
-        glib.Source.__init__(self)
+        gobject.Source.__init__(self)
 
     def prepare(self):
         return True, 0
@@ -29,7 +31,6 @@ class MySource(glib.Source):
     def dispatch(self, callback, args):
         return callback(*args)
 
-
 class TestSource(unittest.TestCase):
     def timeout_callback(self, loop):
         loop.quit()
@@ -39,12 +40,12 @@ class TestSource(unittest.TestCase):
         return True
 
     def setup_timeout(self, loop):
-        timeout = glib.Timeout(500)
+        timeout = gobject.Timeout(500)
         timeout.set_callback(self.timeout_callback, loop)
         timeout.attach()
 
     def testSources(self):
-        loop = glib.MainLoop()
+        loop = gobject.MainLoop()
 
         self.setup_timeout(loop)
 
@@ -61,40 +62,5 @@ class TestSource(unittest.TestCase):
 
         assert self.pos >= 0 and idle.count >= 0
 
-    def testSourcePrepare(self):
-        # this test may not terminate if prepare() is wrapped incorrectly
-        dispatched = [False]
-        loop = glib.MainLoop()
-
-        class CustomTimeout(glib.Source):
-            def prepare(self):
-                return (False, 10)
-
-            def check(self):
-                return True
-
-            def dispatch(self, callback, args):
-                dispatched[0] = True
-
-                loop.quit()
-
-                return False
-
-        source = CustomTimeout()
-
-        source.attach()
-        source.set_callback(dir)
-
-        loop.run()
-
-        assert dispatched[0]
-
-
-class TestTimeout(unittest.TestCase):
-     def test504337(self):
-        timeout_source = glib.Timeout(20)
-        idle_source = glib.Idle()
-
-
 if __name__ == '__main__':
     unittest.main()
